import zope.sqlalchemy
from pyramid.config import Configurator
from pyramid.events import NewRequest
from pyramid.renderers import JSONP
from pyramid.request import Request
from pyramid.response import Response
from pyramid_beaker import BeakerSessionFactoryConfig, session_factory_from_settings
import transaction
import sqlalchemy.orm

from workspaces.capability.services.capability_info import CapabilityInfo
from workspaces.capability.services.capability_service import CapabilityService
from workspaces.notification.services.notification_service import NotificationServiceRESTClient
from workspaces.system.schema import get_engine, get_session_factory
from workspaces.workflow.services.workflow_service import WorkflowServiceRESTClient


# Copied from here: https://stackoverflow.com/questions/21107057/pyramid-cors-for-ajax-requests
def add_cors_headers_response_callback(event: NewRequest):
    """
    Event handler that adds CORS HTTP headers to responses from this server; allows external servers
    (the front end) to send requests and not have them bounce off

    More information about CORS: https://developer.mozilla.org/en-US/docs/Web/HTTP/CORS

    :param event: Server event
    """

    def cors_headers(request: Request, response: Response):
        """
        Callback function that adds CORS headers to a response

        :param response: Server response
        """
        response.headers.update(
            {
                "Access-Control-Allow-Origin": "*",
                "Access-Control-Allow-Methods": "POST,GET,DELETE,PUT,OPTIONS",
                "Access-Control-Allow-Headers": "Origin, Content-Type, Accept, Authorization",
                "Access-Control-Allow-Credentials": "true",
                "Access-Control-Max-Age": "1728000",
                "Content-Type": "application/json",
            }
        )

    event.request.add_response_callback(cors_headers)


# ---------------------------------------------------------
#
#    S E R V I C E S
#
# ---------------------------------------------------------


def add_services(config: Configurator, session_factory: BeakerSessionFactoryConfig) -> Configurator:
    """
    Add capability info, capability service, and workflow service to Pyramid request configuration

    :param config: Pyramid Configurator object
    :param session_factory: Pyramid Beaker session factory
    :return: Updated Configurator
    """
    # # these objects should exist across request boundaries
    workflow_rest_client = WorkflowServiceRESTClient()
    notification_rest_client = NotificationServiceRESTClient()
    capability_info = CapabilityInfo(get_tm_session(session_factory, transaction.manager))
    capability_service = CapabilityService(capability_info, workflow_rest_client, notification_rest_client)

    # the motivation here is to assign the session to the capability info
    # as requests come in
    def attach_session(request):
        capability_info.session = get_tm_session(session_factory, request.tm)
        return capability_info

    # make capability_info available for use in Pyramid
    config.add_request_method(
        # r.tm is the transaction manager used by pyramid_tm
        attach_session,
        "capability_info",
        reify=True,
    )
    # make workflow_info available for use in Pyramid
    config.add_request_method(
        # r.tm is the transaction manager used by pyramid_tm
        lambda request: workflow_rest_client,
        "workflow_service",
        reify=True,
    )
    config.add_request_method(
        lambda request: notification_rest_client,
        "notification_service",
        reify=True
    )
    # make capability_service available for use in Pyramid
    config.add_request_method(
        lambda request: capability_service,
        "capability_service",
        reify=True)
    return config


def lookup_request(request):
    return request.capability_info.lookup_capability_request(request.matchdict["request_id"])


# ---------------------------------------------------------
#
#    F R A M E W O R K    S E T U P
#
# ---------------------------------------------------------


def get_tm_session(
    session_factory: BeakerSessionFactoryConfig, transaction_manager: transaction.TransactionManager
) -> sqlalchemy.orm.Session:
    """
    Enable Zope's transaction manager on our session
    :param session_factory:
    :param transaction_manager:
    :return:
    """
    dbsession = session_factory()
    zope.sqlalchemy.register(dbsession, transaction_manager=transaction_manager)
    return dbsession


# ---------------------------------------------------------
#
#    M A I N    E N T R Y P O I N T
#
# ---------------------------------------------------------


def main(global_config, **settings):
    with Configurator(settings=settings) as config:
        # Helpers
        config.add_subscriber(add_cors_headers_response_callback, NewRequest)

        session_factory = session_factory_from_settings(settings)
        config.set_session_factory(session_factory)
        config.add_renderer("jsonp", JSONP(param_name="callback"))

        settings["tm.manager_hook"] = "pyramid_tm.explicit_manager"

        # use pyramid_tm to hook the transaction lifecycle to the request
        config.include("pyramid_tm")

        # use pyramid_retry to retry a request when transient exceptions occur
        config.include("pyramid_retry")

        session_factory = get_session_factory(get_engine())
        config.registry["dbsession_factory"] = session_factory

        # Add services to config
        config = add_services(config, session_factory)

        config.include("pyramid_beaker")
        # Include routes from routes file
        config.include(".routes")
        # config.scan(".views")
        config.scan(".")
        return config.make_wsgi_app()
