"""
File containing definitions for the capability version routes of the Workspaces REST API
"""
from pyramid.httpexceptions import HTTPBadRequest, HTTPNotFound, HTTPPreconditionFailed
from pyramid.request import Request
from pyramid.response import Response
from pyramid.view import view_config


@view_config(route_name="create_capability_version", renderer="json")
def create_capability_version(request: Request) -> Response:
    """
    Pyramid view that accepts a request to create a capability version
    URL: capability/request/{capability_request_id}/version/create

    :param request: POST request, expecting JSON parameter "parameters"
    :return: 200 OK response with JSON-formatted info of newly created capability request
        or 400 response (HTTPBadRequest) if expected parameters not given
        or 412 response (HTTPPreconditionFailed) if capability request with given id does not exist
    """
    expected_params = ["parameters"]
    capability_request_id = request.matchdict["capability_request_id"]
    params = request.json_body

    if not all([expected in params for expected in expected_params]):
        # JSON params do not contain all expected params
        params_not_given_msg = (
            f"Expected JSON parameters {expected_params}. Received only {params}."
        )
        return HTTPBadRequest(detail=params_not_given_msg)
    elif not request.capability_info.lookup_capability_request(capability_request_id):
        # CapabilityRequest with given id does not exist; can't create a new version for it
        does_not_exist_msg = (
            f"Capability request #{capability_request_id} does not exist. Cannot create new version",
        )
        return HTTPPreconditionFailed(detail=does_not_exist_msg)
    else:
        new_capability_version = request.capability_service.create_new_version(
            capability_request_id, parameters=params["parameters"]
        )
        return Response(json_body=new_capability_version.__json__())


@view_config(route_name="add_version_file", renderer="json")
def add_version_file(request: Request) -> Response:
    """
    Pyramid view that accepts a request to add a file to a capability version
    URL: capability/request/{capability_request_id}/version/file/add

    :param request: POST request, expecting JSON parameter "parameters"
           containing "filename" and "content" of the file to add
    :return: 200 OK response with JSON-formatted info of newly created capability request
        or 400 response (HTTPBadRequest) if expected parameters not given
        or 412 response (HTTPPreconditionFailed) if capability version with given request id does not exist
    """
    expected_params = ["parameters"]
    capability_request_id = request.matchdict["capability_request_id"]
    params = request.json_body

    if not all([expected in params for expected in expected_params]):
        # JSON params do not contain all expected params
        params_not_given_msg = (
            f"Expected JSON parameters {expected_params}. Received only {params}."
        )
        return HTTPBadRequest(detail=params_not_given_msg)
    elif not request.capability_info.lookup_capability_request(capability_request_id):
        # CapabilityVersion with given request id and version number does not exist; can't add files
        does_not_exist_msg = (
            f"Capability request #{capability_request_id} does not exist. Cannot add file.",
        )
        return HTTPPreconditionFailed(detail=does_not_exist_msg)
    else:
        file_input = params["parameters"]
        current_version = request.capability_info.lookup_current_version(capability_request_id)
        file = request.capability_info.save_version_file(
            version=current_version,
            filename=file_input["filename"],
            content=file_input["content"].encode('ascii')
        )
        return Response(json_body=file.__repr__())
